@@ -14,11 +14,11 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
-// authutils contains small utility functions without much logic
+// Package utils contains small utility functions without much logic
 // wrapping the major APIs of the core auth package for ease of use
 // in the controllers. These functions also import the provider
 // packages to wrap switch-case choice of provider implementations.
 // Because of that, these functions cannot be placed in the core
 // package as they would cause a cyclic dependency given that the
 // provider packages also import the core package.
-package authutils
+package utils
