@@ -217,6 +217,15 @@ some:
 					dyff.REMOVAL, yml(`version: v1`), nil,
 					dyff.ADDITION, nil, yml(`release: v1`))))
 			})
+
+			It("should ignore leading and trailing whitespace changes if configured", func() {
+				from := yml(`{"foo": "bar"}`)
+				to := yml(`{"foo": "bar "}`)
+
+				diffs, err := compare(from, to, dyff.IgnoreWhitespaceChanges(true))
+				Expect(err).To(BeNil())
+				Expect(diffs).To(BeNil())
+			})
 		})
 
 		Context("Given two YAML structures with simple lists", func() {
@@ -822,6 +831,19 @@ listY: [ Yo, Yo, Yo ]
 
 				Expect(report.ExcludeRegexp("/does/not/exist")).To(BeEquivalentTo(report))
 			})
+
+			It("should ignore changes in values", func() {
+				report := dyff.Report{Diffs: []dyff.Diff{
+					singleDiff("/yaml/map/add", dyff.ADDITION, nil, "added"),
+					singleDiff("/yaml/map/removed", dyff.REMOVAL, nil, "removed"),
+					singleDiff("/yaml/map/changed", dyff.MODIFICATION, "foobar", "barfoo"),
+				}}
+
+				Expect(report.IgnoreValueChanges()).To(BeEquivalentTo(dyff.Report{Diffs: []dyff.Diff{
+					singleDiff("/yaml/map/add", dyff.ADDITION, nil, "added"),
+					singleDiff("/yaml/map/removed", dyff.REMOVAL, nil, "removed"),
+				}}))
+			})
 		})
 
 		Context("change root for comparison", func() {
