@@ -23,6 +23,7 @@ import (
 	"io"
 	"net"
 	"net/http"
+	"strings"
 	"time"
 
 	"github.com/hashicorp/go-retryablehttp"
@@ -37,11 +38,11 @@ type (
 		DeleteUser(user int, options ...RequestOptionFunc) (*Response, error)
 		CurrentUser(options ...RequestOptionFunc) (*User, *Response, error)
 		CurrentUserStatus(options ...RequestOptionFunc) (*UserStatus, *Response, error)
-		GetUserStatus(user int, options ...RequestOptionFunc) (*UserStatus, *Response, error)
+		GetUserStatus(uid any, options ...RequestOptionFunc) (*UserStatus, *Response, error)
 		SetUserStatus(opt *UserStatusOptions, options ...RequestOptionFunc) (*UserStatus, *Response, error)
 		GetUserAssociationsCount(user int, options ...RequestOptionFunc) (*UserAssociationsCount, *Response, error)
 		ListSSHKeys(opt *ListSSHKeysOptions, options ...RequestOptionFunc) ([]*SSHKey, *Response, error)
-		ListSSHKeysForUser(uid interface{}, opt *ListSSHKeysForUserOptions, options ...RequestOptionFunc) ([]*SSHKey, *Response, error)
+		ListSSHKeysForUser(uid any, opt *ListSSHKeysForUserOptions, options ...RequestOptionFunc) ([]*SSHKey, *Response, error)
 		GetSSHKey(key int, options ...RequestOptionFunc) (*SSHKey, *Response, error)
 		GetSSHKeyForUser(user int, key int, options ...RequestOptionFunc) (*SSHKey, *Response, error)
 		AddSSHKey(opt *AddSSHKeyOptions, options ...RequestOptionFunc) (*SSHKey, *Response, error)
@@ -87,7 +88,7 @@ type (
 		DeleteUserIdentity(user int, provider string, options ...RequestOptionFunc) (*Response, error)
 
 		// events.go
-		ListUserContributionEvents(uid interface{}, opt *ListContributionEventsOptions, options ...RequestOptionFunc) ([]*ContributionEvent, *Response, error)
+		ListUserContributionEvents(uid any, opt *ListContributionEventsOptions, options ...RequestOptionFunc) ([]*ContributionEvent, *Response, error)
 	}
 
 	// UsersService handles communication with the user related methods of
@@ -168,6 +169,7 @@ type User struct {
 	AvatarURL                      string             `json:"avatar_url"`
 	CanCreateGroup                 bool               `json:"can_create_group"`
 	CanCreateProject               bool               `json:"can_create_project"`
+	CanCreateOrganization          bool               `json:"can_create_organization"`
 	ProjectsLimit                  int                `json:"projects_limit"`
 	CurrentSignInAt                *time.Time         `json:"current_sign_in_at"`
 	CurrentSignInIP                *net.IP            `json:"current_sign_in_ip"`
@@ -457,10 +459,11 @@ func (s *UsersService) CurrentUser(options ...RequestOptionFunc) (*User, *Respon
 // GitLab API docs:
 // https://docs.gitlab.com/api/users/#get-your-user-status
 type UserStatus struct {
-	Emoji        string            `json:"emoji"`
-	Availability AvailabilityValue `json:"availability"`
-	Message      string            `json:"message"`
-	MessageHTML  string            `json:"message_html"`
+	Emoji         string            `json:"emoji"`
+	Availability  AvailabilityValue `json:"availability"`
+	Message       string            `json:"message"`
+	MessageHTML   string            `json:"message_html"`
+	ClearStatusAt *time.Time        `json:"clear_status_at"`
 }
 
 // CurrentUserStatus retrieves the user status
@@ -482,12 +485,20 @@ func (s *UsersService) CurrentUserStatus(options ...RequestOptionFunc) (*UserSta
 	return status, resp, nil
 }
 
-// GetUserStatus retrieves a user's status
+// GetUserStatus retrieves a user's status.
+//
+// uid can be either a user ID (int) or a username (string); will trim one "@" character off the username, if present.
+// Other types will cause an error to be returned.
 //
 // GitLab API docs:
 // https://docs.gitlab.com/api/users/#get-the-status-of-a-user
-func (s *UsersService) GetUserStatus(user int, options ...RequestOptionFunc) (*UserStatus, *Response, error) {
-	u := fmt.Sprintf("users/%d/status", user)
+func (s *UsersService) GetUserStatus(uid any, options ...RequestOptionFunc) (*UserStatus, *Response, error) {
+	user, err := parseID(uid)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	u := fmt.Sprintf("users/%s/status", strings.TrimPrefix(user, "@"))
 
 	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
 	if err != nil {
@@ -508,9 +519,10 @@ func (s *UsersService) GetUserStatus(user int, options ...RequestOptionFunc) (*U
 // GitLab API docs:
 // https://docs.gitlab.com/api/users/#set-your-user-status
 type UserStatusOptions struct {
-	Emoji        *string            `url:"emoji,omitempty" json:"emoji,omitempty"`
-	Availability *AvailabilityValue `url:"availability,omitempty" json:"availability,omitempty"`
-	Message      *string            `url:"message,omitempty" json:"message,omitempty"`
+	Emoji            *string                `url:"emoji,omitempty" json:"emoji,omitempty"`
+	Availability     *AvailabilityValue     `url:"availability,omitempty" json:"availability,omitempty"`
+	Message          *string                `url:"message,omitempty" json:"message,omitempty"`
+	ClearStatusAfter *ClearStatusAfterValue `url:"clear_status_after,omitempty" json:"clear_status_after,omitempty"`
 }
 
 // SetUserStatus sets the user's status
@@ -609,7 +621,7 @@ type ListSSHKeysForUserOptions ListOptions
 //
 // GitLab API docs:
 // https://docs.gitlab.com/api/user_keys/#list-all-ssh-keys-for-a-user
-func (s *UsersService) ListSSHKeysForUser(uid interface{}, opt *ListSSHKeysForUserOptions, options ...RequestOptionFunc) ([]*SSHKey, *Response, error) {
+func (s *UsersService) ListSSHKeysForUser(uid any, opt *ListSSHKeysForUserOptions, options ...RequestOptionFunc) ([]*SSHKey, *Response, error) {
 	user, err := parseID(uid)
 	if err != nil {
 		return nil, nil, err
@@ -1490,7 +1502,7 @@ type UserActivity struct {
 
 // GetUserActivitiesOptions represents the options for GetUserActivities
 //
-// GitLap API docs:
+// GitLab API docs:
 // https://docs.gitlab.com/api/users/#list-a-users-activity
 type GetUserActivitiesOptions struct {
 	ListOptions
